@@ -11,10 +11,11 @@ let md5_of_file =
 let () =
   if md5 <> md5_of_file then (
     Printf.eprintf
-      "MD5 for %s differ:\n\
+      "MD5 for %s differs:\n\
       \  expected: %s\n\
       \    actual: %s\n"
       file md5 md5_of_file;
-    Sys.remove file
+    Sys.remove file;
+    exit 1
   ) else
     Printf.printf "%s has the expected MD5.\n" file
